<?php
// $Id: customfilter.export.inc,v 1.1.2.9 2009/06/21 08:20:56 kiam Exp $

/**
 * @file
 * Exports filters into XML content.
 */

/***************************************************************************** 
 * Menu callbacks / form builders, submit/validate functions.
 ****************************************************************************/

/**
 * Return the export form.
 */
function customfilter_export_form() {
  $form = array();
  
  if ($filters = customfilter_get_filters()) {
    foreach ($filters as $filter) {
      $opt[$filter['fid']] = t($filter['name']);
    }
    
    $form['filters'] = array(
      '#type' => count($filters) > 11 ? 'select' : 'checkboxes',
      '#title' => t('Filters'),
      '#options' => $opt,
      '#description' => 'Choose the filters to export.',
    );
    if (count($filters) > 11) {
      $form['filters']['#multiple'] = TRUE;
    }
    
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Export',
    );
  }
  else {
    $form['filters'] = array(
      '#type' => 'markup',
      '#value' => '<p>'. t('There are no custom filters defined.') .'</p>',
    );
  }
  
  return $form;
}
 
function customfilter_export_form_submit($form, &$form_state) {
  $fids = array();
  
  foreach ($form_state['values']['filters'] as $fid) {
    if ($fid > 0) {
      $fids[] = $fid;
    }
  }
  
  $form_state['redirect'] = 'admin/settings/filters/customfilter/export/xml/'. implode('.', $fids);
}

function customfilter_xml_export($sfids) {
  $fids = explode('.', $sfids);
  
  $xml[] = '<?xml version="1.0" standalone="yes"?>';
  $xml[] = '<customfilter>';
  
  foreach ($fids as $fid) {
    $filter = customfilter_get_filter($fid, '*');  
    $rules = customfilter_get_rules($fid, 0);
    
    $xml[] = '  <filterset name="'. $filter['name'] .' cache="'. $filter['cache'] .'">';
    $xml[] = '    <description>'. $filter['description'] .'</description>';
    $xml[] = '    <tips>';
    $xml[] = '      <short><![CDATA['. $filter['shorttips'] .']]></short>';
    $xml[] = '      <long><![CDATA['. $filter['longtips'] .']]></long>';
    $xml[] = '    </tips>';
    $xml[] = '    <rules>';
    
    foreach ($rules as $rule) {
      $xml[] = _customfilter_xml_rule($rule);
    }
    
    $xml[] = '    </rules>';
    $xml[] = '  </filterset>';
  }
  
  $xml[] = '</customfilter>';
  
  drupal_header('Content-Type: application/octet-stream');
  drupal_header('Content-Length: '. strlen($xml));
  drupal_header("Content-Disposition: attachment; filename=customfilter.xml");
  
  echo implode("\n", $xml);
}

/**
 * Return the import form.
 */
function customfilter_import_form() {
  $form = array();
  
  $form['xml'] = array(
    '#type' => 'file',
    '#title' => t('Import from file'),
    '#description' => t('The XML file to be imported.'),
  );
  
  $form[] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );
  
  $form['#attributes'] = array('enctype' => 'multipart/form-data');
  
  return $form;
}
 
function customfilter_import_form_submit($form, &$form_state) {
  $globals =& customfilter_globals();
  
  $file = file_save_upload('xml');
  
  if ($file) {
    if ($thefile = fopen($file->filepath, "rb")) {
      $fstat = fstat($thefile);
      $xml = fread($thefile, $fstat['size']);
      
      fclose($thefile);
      drupal_set_message(t('XML file %f imported.', array('%f' => $file->filename)));
      
      customfilter_xml_read($xml);
      _customfilter_xml_sql($globals->filters, $globals->rules);
    }
  }
}

function customfilter_xml_read(&$xml) {
  $globals =& customfilter_globals();
  
  $globals->elements = array();
  $globals->filters = array();
  $globals->rules = array();
  $globals->filter = array();
  $globals->rule = array();
  $globals->parents = array();
  $globals->fid = 0;
  $globals->rid = 0;
  
  $parser = drupal_xml_parser_create($xml);
  xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, true);
  xml_set_element_handler($parser, '_customfilter_xml_element_start', '_customfilter_xml_element_end');
  xml_set_character_data_handler($parser, '_customfilter_xml_element_cdata');
  xml_parse($parser, $xml, TRUE);
  xml_parser_free($parser);
}
 
/*****************************************************************************
 * Private functions.
 ****************************************************************************/

function _customfilter_xml_element_start($parser, $name, $attr) {
  $globals =& customfilter_globals();
  
  $globals->elements[] = $name;
  switch ($name) {
    case 'FILTERSET':
      $globals->filter['fid'] = $globals->fid;
      $globals->filter['name'] = $attr['NAME'];
      $globals->filter['cache'] = $attr['CACHE'];
      $globals->filter['description'] = '';
      $globals->filter['shorttips'] = '';
      $globals->filter['longtips'] = '';
      $globals->fid++;
      break;
    
    case 'FILTER':
    case 'RULE':
      $globals->rule['rid'] = $globals->rid;
      $globals->rule['fid'] = $globals->filter['fid'];
      $globals->rule['name'] = $attr['NAME'];
      $globals->rule['matches'] = $attr['MATCHES'];
      $globals->rule['weight'] = $attr['WEIGHT'];
      $globals->rule['description'] = '';
      $globals->rule['pattern'] = '';
      $globals->rule['replacement'] = '';
      $globals->rid++;
      
      if ($attr['function']) {
        $globals->rule['code'] = $attr['function'];
      }
      elseif ($attr['CODE']) {
        $globals->rule['code'] = $attr['CODE'];
      }
      
      if (count($globals->parents)) {
        $parent = end($globals->parents);
        $globals->rule['prid'] = $parent['rid'];
      }
      else {
        $globals->rule['prid'] = 0;
      }
      break;
    
    case 'SUBFILTERS':
    case 'SUBRULES':
      $globals->parents[] = $globals->rule;
      $globals->rule = array();
      break;
    case 'FILTERS':
    case 'RULES':
      $globals->parents = array();
      $globals->rule = array();
      break;
  }
}

function _customfilter_xml_element_end($parser, $name) {
  $globals =& customfilter_globals();
  
  switch ($name) {
    case 'FILTERSET':
      $globals->filters[$globals->filter['fid']] = $globals->filter;
      $globals->filter = array();
      break;
    
    case 'FILTER':
    case 'RULE':
      $globals->rules[$globals->rule['rid']] = $globals->rule;
      $globals->rule = array();
      break;
    
    case 'SUBFILTERS':
    case 'SUBRULES':
      $globals->rule = array_pop($globals->parents);
      break;
  }
  
  array_pop($globals->elements);
}

function _customfilter_xml_element_cdata($parser, $data) {
  $globals =& customfilter_globals();
  
  $element = end($globals->elements);
  $context = prev($globals->elements);
  
  switch ($element) {
    case 'DESCRIPTION':
      if ($context == 'FILTERSET') {
        $globals->filter['description'] = $data; 
      }
      elseif ($context == 'FILTER' || $context == 'RULE') {
        $globals->rule['description'] = $data; 
      }
      break;
    
    case 'SHORT':
    case 'LONG':
      $globals->filter[drupal_strtolower($element) .'tips'] = $data;
      break;
    
    case 'PATTERN':
    case 'REPLACEMENT':
      $globals->rule[drupal_strtolower($element)] = $data;
      break;  
  }
}

function _customfilter_xml_rule($rule, $level = 0) {
  $indent = str_repeat('  ', $level + 3);
  
  $xml[] = $indent .'<rule name="'. $rule['name'] .'" matches="'. $rule['matches'].'" code="'. $rule['code'] .'" weight="'. $rule['weight'] .'"';
  $xml[] = $indent .'  <description><![CDATA['. $rule['description'] .']]></description>';
  $xml[] = $indent .'  <pattern><![CDATA['. $rule['pattern'] .']]></pattern>';
  $xml[] = $indent .'  <replacement><![CDATA['. $rule['replacement'] .']]></replacement>';
  
  if (is_array($subrules = $rule['sub']) && count($subrules)) {
    $xml[] = $indent .'  <subrules>';
    foreach ($subrules as $subrule) {
      $xml[] = _customfilter_xml_rule($subrule, $level + 2);
    }
    $xml[] = $indent .'  </subrules>';
  }
  
  $xml[] = $indent .'</rule>';
  
  return implode("\n", $xml);
}

function _customfilter_xml_sql($filters, $rules) {
  $fidmap = array();
  $ridmap = array();
  
  foreach ($filters as $filter) {
    $prevfid = $filter['fid'];
    
    drupal_write_record('customfilter_filters', $filter);
    
    $fidmap[$prevfid] = $filter['fid'];
  }
  
  ksort($rules);
  
  foreach ($rules as $rule) {
    $prevrid = $rule['rid'];
    
    $rule['fid'] = $fidmap[$rule['fid']];
    if ($rule['prid'] > 0) {
      $rule['prid'] = $ridmap[$rule['prid']];
    }
    
    drupal_write_record('customfilter_rules', $rule);
    
    $ridmap[$prevrid] = $rule['rid'];
  }
}
